#-----------------------------------------------------------------------------
#
#   Loon - Base class for button-like menus
#
#-----------------------------------------------------------------------------

from pygame import Color
from albow.utils import blit_in_rect, frame_rect
from albow.palette_view import PaletteView

class ButtonMenu(PaletteView):

	bg_color = Color("lightblue")
	highlight_color = Color("orange")
	item_border_color = Color("black")
	item_border_width = 2
	item_inset = 2
	clicked_item = None
	clicked_rect = None
	in_clicked_item = False

	def draw_item(self, surf, i, rect):
		d = 2 * self.item_inset
		rect = rect.inflate(-d, -d)
		if i == self.clicked_item and self.in_clicked_item:
			surf.fill(self.highlight_color, rect)
		frame_rect(surf, self.item_border_color, rect, self.item_border_width)
		text = self.item_text(i)
		buf = self.font.render(text, True, self.fg_color)
		blit_in_rect(surf, buf, rect, 'c')

	def click_item(self, i, e):
		self.clicked_item = i
		self.clicked_rect = self.item_rect(i)
		self.in_clicked_item = True
	
	def mouse_drag(self, e):
		r = self.clicked_rect
		h = r is not None and r.collidepoint(e.local)
		if self.in_clicked_item <> h:
			self.in_clicked_item = h
			self.invalidate()

	def mouse_up(self, e):
		if self.in_clicked_item:
			self.invoke_item(self.clicked_item)
		self.clicked_item = None
		self.clicked_rect = None
		self.in_clicked_item = False
	
	def item_rect(self, i):
		nr = self.num_rows()
		return self.cell_rect(i // nr, i % nr)

