#-----------------------------------------------------------------------------
#
#   Loon - Base class for factory units
#
#-----------------------------------------------------------------------------

from __future__ import division
from random import uniform
from albow.resource import get_image
from unit import Unit
from resources import ResourcePack
from constants import *
import game

class FactoryUnit(Unit):

	valid_multi_commands = {'build'}

	building_unit_class = None
	is_building = False
	build_progress = 0
	missing_resources = None
	
	def is_busy(self):
		return self.is_building
	
	@property
	def build_progress_fraction(self):
		cls = self.building_unit_class
		if cls:
			return self.build_progress / cls.time_to_build()
	
	def command_available(self, name):
		if name == 'build':
			return self.building_unit_class is None
		else:
			return Unit.command_available(name)

	def build_unit(self, client, unit_class):
		if self.building_unit_class is None:
			self.building_unit_class = unit_class
			self.is_building = False
			self.build_progress = 0
			self.missing_resources = None
			client.world.invalidate_unit_commands()
			client.enter_unit_command_mode()

	def live_timestep(self, dt):
		cls = self.building_unit_class
		if cls:
			if not self.is_building:
				self.try_to_start_building(cls)
			else:
				if self.build_progress >= cls.time_to_build():
					self.finish_building()
				else:
					self.build_progress += dt

	def try_to_start_building(self, cls):
		missing = self.resources.use(**cls.construction_resources)
		if missing:
			self.missing_resources = missing
		else:
			self.missing_resources = None
			self.is_building = True

	def finish_building(self):
		unit = self.building_unit_class()
		r = self.radius + unit.radius + 0.5
		p = self.position + (uniform(-r, r), r)
		world = game.game.world
		world.add_unit(unit, p)
		world.invalidate_unit_commands()
		self.building_unit_class = None
		self.is_building = None

	def activity_description(self):
		cls = self.building_unit_class
		if cls:
			return ("Building %s (%.0f%%)" % (cls.title, 100 * self.build_progress_fraction),)

	def display_activity(self, status_display):
		Unit.display_activity(self, status_display)
		if self.building_unit_class:
			missing = self.missing_resources
			if missing:
				descriptions = ["%.2g %s" % (amount, name) for (name, amount) in missing.iteritems()]
				status_display.put("Missing " + ", ".join(descriptions))
				status_display.leave_gap()
