#-----------------------------------------------------------------------------
#
#   Loon - Mobile unit
#
#-----------------------------------------------------------------------------

from math import pi, atan2
from numpy import rint
from unit import Unit
from vector_algebra import mag
import missions

class MobileUnit(Unit):
	#  movement_speed   float        world units per second
	#  num_angles       int          number of viewing angles
	#  num_phases       int          number of movement animation phases
	#  angle            float        current orientation in radians
	#  phase            int          current movement animation phase
	#  images           ImageArray   animation frames
	
	movement_speed = 1.0
	obstructing = False
	is_fixed = False
	
	num_angles = 1
	num_phases = 1
	angle = 0
	phase = 0
	
	def get_image(self):
		n = self.num_angles
		a = rint(self.angle / (2 * pi / n)) % n
		if self.num_phases > 1:
			return self.images[a, self.phase]
		else:
			return self.images[a]
	
	def live_timestep(self, dt):
		#print "MobileUnit.live_timestep:", dt ###
		Unit.live_timestep(self, dt)
		mission = self.mission
		if mission:
			mission.carry_out(self, dt)
		
	def move_towards(self, target, dt):
		#  Take a step towards the specified unit or position.
		#  Returns true if the target has been reached.
		#print "MobileUnit.move_towards:", target ###
		if isinstance(target, Unit):
			pt = target.position
			d0 = self.radius + target.radius
		else:
			pt = target
			d0 = 0.1
		p0 = self.position
		dp = pt - p0
		d = mag(dp)
		if d <= d0:
			#if not isinstance(target, Unit):
			#	self.target = None
			self.phase = 0
			#self.arrived_at_target()
			return True
		else:
			s = min(d, self.movement_speed * dt)
			self.position = p0 + dp * (s / d)
			self.angle = atan2(dp[1], dp[0])
			self.phase = (self.phase + 1) % self.num_phases
			return False
	
	def activity_description(self):
		mission = self.mission
		if mission:
			#return ("Mission", mission.get_goal_description())
			return (mission.get_goal_description(),)
	
	def assign_mission(self, mission):
		mission.begin(self)
