#-----------------------------------------------------------------------------
#
#   Loon - Unit command menu widget
#
#-----------------------------------------------------------------------------

from pygame import Color
from albow.utils import blit_in_rect, frame_rect
from albow.palette_view import PaletteView
from button_menu import ButtonMenu
from action_widget import ActionWidget
from game import game

class UnitCommandMenu(ButtonMenu, ActionWidget):

	button_size = (96, 32)
	
	@property
	def world(self):
		return game.world
	
	@property
	def valid_commands(self):
		return self.world.valid_unit_commands

	def __init__(self, client):
		ButtonMenu.__init__(self, self.button_size, 3, 3)
		self.client = client
	
	def num_items(self):
		return len(self.valid_commands)
	
	def item_text(self, item):
		cmd = self.valid_commands[item]
		text = cmd.title
		key = cmd.key_name
		if key:
			text = "%s (%s)" % (text, key)
		return text
	
	def invoke_item(self, item):
		cmd = self.valid_commands[self.clicked_item]
		self.invoke_command(cmd)
	
	def invoke_command(self, cmd):
		self.client.set_current_command(cmd)
		cmd.invoke(self.client)

	def world_click(self, e):
		self.client.perform_unit_selection(e)

	def key_down(self, e):
		key = e.key
		for cmd in self.valid_commands:
			if cmd.key == key:
				self.invoke_command(cmd)
				return
		PaletteView.key_down(self, e)
