#-----------------------------------------------------------------------------
#
#   Loon - Resource Deposit
#
#-----------------------------------------------------------------------------

from __future__ import division
from math import pi, floor, ceil, sqrt, hypot
from random import randrange, choice, uniform, normalvariate
from albow.resource import get_image
from unit import Unit
from vector_algebra import vector

debug_deposits = False

#-----------------------------------------------------------------------------

class ResourceDeposit(Unit):
	#  resource_type   string
	
	def __init__(self, name, amount):
		self.resource_type = name
		self.health = amount
		self.max_health = amount

	def get_image(self):
		return deposit_images[self.resource_type]
	
	@property
	def title(self):
		return self.health_label + " deposit"
	
	@property
	def health_label(self):
		return self.resource_type.title()

#-----------------------------------------------------------------------------

deposit_types = ['ore', 'water', 'silica']
deposit_abundance= 0.05
mean_cluster_size = 12
sd_cluster_size = 8
csd_num_clusters = 0.1
deposit_size = 10

deposit_image_names = dict(
	ore = "deposit_ore.png",
	water = "deposit_water.png",
	silica = "deposit_silica.png",
)

deposit_images = {name : get_image(deposit_image_names[name]) for name in deposit_image_names}

def add_deposits_to_world(world):
	terrain = world.terrain
	w, h = terrain.shape
	#w, h = 10, 10 ###
	mean_num_clusters = deposit_abundance * w * h / mean_cluster_size;
	sd_num_clusters = csd_num_clusters * mean_num_clusters;
	num_clusters = int(ceil(normalvariate(mean_num_clusters, sd_num_clusters)))
	if debug_deposits:
		print "add_deposits_to_world: Adding %s clusters" % num_clusters
	for i in xrange(num_clusters):
		xc = uniform(0, w)
		yc = uniform(0, h)
		cluster_size = max(1, normalvariate(mean_cluster_size, sd_cluster_size))
		cluster_type = choice(deposit_types)
		if debug_deposits:
			print "add_deposits_to_world: Adding cluster of %s sized %s at %s" % (cluster_type, cluster_size, [xc, yc])
		rsq = cluster_size / pi
		r = ceil(sqrt(rsq))
		x0 = int(floor(xc - r))
		x1 = int(ceil(xc + r))
		y0 = int(floor(yc - r))
		y1 = int(ceil(yc + r))
		if debug_deposits:
			print "add_deposits_to_world: x0 = %s, x1 = %s, y0 = %s, y1 = %s" % (x0, x1, y0, y1)
		for x in xrange(x0, x1 + 1):
			for y in xrange(y0, y1 + 1):
				if 0 <= x < w and 0 <= y < h:
					if hypot(x + 0.5 - xc, y + 0.5 - yc) <= r:
						cell = terrain[x, y]
						if cell.unit is None:
							if debug_deposits:
								print "add_deposits_to_world: Adding deposit at %s" % [x, y]
							unit = ResourceDeposit(cluster_type, deposit_size)
							world.add_unit(unit, vector(x + 0.5, y + 0.5))
